SeaIceModes <- function(var, exp, obs, sdates, seasoninf, seasonsup, nclusters = 4, nleadtime = NULL, nmember = NULL, leadtimemin = 1, leadtimemax = NULL, storefreq = "monthly", sampleperiod = 1, detrended = FALSE, nemoversion = N3.2_O1L42 ) {
  #
  # This function computes sea ice modes of variability based on the k-mean
  # method applied to sea ice area or extent averages over the Arctic Seas.
  #
  # Args:
  #   var can be sia or sie or sit (sea ice area or extent or thickness)
  #   exp, obs, sdates, nleadtime, nmember, leadtimemin, leadtimemax, 
  #            storefreq are sampleperiod are the same as in Load;
  #            only exp, obs and sdates are compulsory
  #   seasoninf is the month of the year month when to start computing the 
  #            extended seasonal means: 1 to 12.
  #   seasonsup is the month of the year when to stop computing the extended
  #            seasonal means: 1 to 12. The extended seasonal means can overlap
  #            the 31s December: DJF average for example.
  #   nclusters : number of clusters to be computed
  #
  # Return:
  #   clusters : 3-dimensional matrice (x, y, nclusters) containing the
  #              centroides  
  #   pvalue   : 3-dimensional matrice (x, y, nclusters) containing the
  #              pvalue of the centroide anomalies obtained through a t-test
  #              that accounts for the serial dependence of the data
  #   occurrences : 2-dimension matrice (nclusters, time) containing 1 (when
  #                 a given cluster occurs) or 0 (when it does not occur)
  #
  # History:
  #
###############################################################################

# This is listseas of non-overlapping regions in the NH (default of Arcticreg1) where sea ice can appear at least in winter.
# Also, regional masks are designed to include only grid points where sea ice appears only at least once in the entire record.

listseas=c( 'Baffin_Bay', 'Baltic_Sea', 'Barents_Sea', 'Beaufort_Sea', 'Bering', 'CanArch', 'Chukchi_Sea', 'CntArctic', 'CntArcticRing1', 'CntArcticRing2', 'CntArcticRing3', 'CntArcticRing4', 'CntArcticRing5', 'CntArcticRing6', 'CntArcticRing7_Lincoln_Sea', 'CntArcticRing8', 'CntArcticPrf1', 'CntArcticPrf2r', 'CntArcticPrf3', 'CntArcticPrf4', 'East_Siberian_Sea1', 'Greenland_Sea', 'Hudson', 'Icelandic_Sea', 'Irminger', 'Japan1', 'Kara_Sea', 'Laptev_Sea', 'Labrador_Sea1', 'Norwegian_Sea', 'Okhotsk', 'StLawr' )

# 29/07 : The areas should not be hard-coded. If you use the NSIDC or HadISST dataset,
# 29/07 : the areas would actually be different. So you need a call to a generic
# 29/07 : function here that computes the weight for each sea according to 
# 29/07 : a grid description flag. It could be nemoversion=N3.2_O1L42, in
# 29/07 : which case your function read the following files:
# 29/07 : /cfu/autosubmit/con_files/new_maskglo.N3.2_O1L42.nc
# 29/07 : /cfu/autosubmit/con_files/mask.regions.N3.2_O1L42.nc
# 29/07 : can compute the weight for this configuration. This is important
# 29/07 : because in about one month I will give you other reconstructions 
# 29/07 : with LIM3 and nemoversion = Ec3.0_O1L46 and later you can try
# 29/07 : on the UCL reanalysis or the high resolution LIM3 reconstructions
# 29/07 : also for HadISST and NSIDC, you should read the latitude and 
# 29/07 : can compute the weights from the list of latitude

weights <- rep(1,length(listseas))

if ( var != 'sit' ) {
   filename1 <- paste('/cfu/autosubmit/con_files/area_tmask_nemo.',nemoversion,'.nc',sep='')
   ncid1     <- open.ncdf(filename1)
   data1     <- get.var.ncdf(ncid1,'AREA')

   filename2 <- paste('/cfu/autosubmit/con_files/mask.regions.',nemoversion,'.',exp,'.nc',sep='')
   ncid2     <- open.ncdf(filename2)
}

for (sea in listseas) {
    tmp=Load(paste(var,sea,sep='_'),exp,obs,sdates,nleadtime=nleadtime,nmember=nmember,leadtimemin=leadtimemin,leadtimemax=leadtimemax,storefreq=storefreq,sampleperiod=sampleperiod)  

    if ( exp != NULL ) {
       mod1 <- Season(tmp$mod,posdim=4,monini=as.integer(substr(sdates,5,6)),moninf=seasoninf,monsup=seasonsup)
       mod2 <- Mean1Dim(mod1,2)

       if (listseas[1] == sea) { 
          model <- array(dim=c(length(mod2[1,1,]),length(listseas)))
          ii <- 1
          model[,ii] <- mod2[1,1,]
       } else { 
         ii <- ii+1
         model[,ii] <- mod2[1,1,]
       }
    }

    if ( obs != NULL ) {
       obs1 <- Season(tmp$obs,posdim=4,monini=as.integer(substr(sdates,5,6)),moninf=seasoninf,monsup=seasonsup)
       obs2 <- Mean1Dim(obs1,2)

       if (listseas[1] == sea) { 
          obser <- array(dim=c(length(obs2[1,1,]),length(listseas)))
          ii <- 1
          obser[,ii] <- obs2[1,1,]
       } else { 
         ii <- ii+1
         obser[,ii] <- obs2[1,1,]
       }
    }

    if ( var != 'sit' ) { 
       varname2 <- sea 
       data2 <- get.var.ncdf(ncid2,varname2)
       weights[ii] <- sum(data1*data2)/(10^9)
    }
}

# Once you have recoded the lines above and you have made a few tests on the 
# computation of clusters, you can work on adding the step of computing
# the weights to be sent to Cluster at this location of the function. For now,
# I have set weights to 1 as a default so that they don't bother you for the
# first tests.

# Then here, you send saga to the Cluster function
# You obtain the array of occurrences that you can use to composite the sea ice
# concentration to obtain clusters with dimensions (x, y, time). To do so,
# you need to call Load('sic', wil all the arguments used when loading the area
# averages)
#
# Then you call the function Composite to obtain the clusters on the (x,y,time)
# grid and their pvalue
#
 
if ( exp != null ) {
   if ( detrended == FALSE ) { model1 <- Cluster(model,weights,nclusters=NULL) }
      else { model1 <- Cluster(Trend(model,1)$detrended,weights,nclusters=NULL) } 

# 5 lines below are only temporary used in testing phase
   save(model,file='mod_input_to_Cluster.sav')
   print(' ')
   print('Mod. time series of occurance is')
   print(' ')
   print(model1$cluster)
}

if ( obs != null ) {
   if ( detrended == FALSE ) { obser1 <- Cluster(obser,weights,nclusters=NULL) }
      else { obser1 <- Cluster(Trend(obser,1)$detrended,weights,nclusters=NULL) } 

# 5 lines below are only temporary used in testing phase
   save(obser,file='obs_input_to_Cluster.sav')
   print(' ')
   print('Obs. time series of occurance is')
   print(' ')
   print(obser1$cluster)
}

# this is not the complete output of SeaIceModes, but a version used in testing phase
invisible(model1)

}
